/* @Copyright OCHIAI LAB. of Nara Women's University
programmed by Miss Kiwada and M. Ochiai by Hopcroft-Tarjan algorithm
1994-2004
*/

#include  <stdio.h>
#include  <stdlib.h>
#include  <setjmp.h>
#include  <math.h>
#include  <cstdlib> 

#define maxv   200
#define maxv2  400
#define maxe   500

#define	max(x,y)	((x > y) ? x: y)
#define	min(x,y)	((x < y) ? x: y)

typedef struct link {
        int    value;
        struct link *next;
} link;

typedef struct link2 {
        int    lt, rt;
        struct link2 *next;
} link2;

extern FILE			*infile;
extern char			filename[]; /* *filename no good */
extern int			INDATA;

static FILE			*outfile = NULL;

static jmp_buf		env;
static int			retcode;

static int			g_nv, /* vertex nu(G)*/
					g_ne, /* edge   epsilon(G) */
					g_k, g_ct;
static link			*free1; 
static link2		*free2, *temp2, *est, *bst; 
static int			orgl[maxv], no[maxv], lpt1[maxv], lpt2[maxv];
static int			pos = 0, pos2 = 0; 
static link			*a[maxv], *pa[maxv], *listlink[maxv2];
static link2		*bt[maxv2], *listlink2[maxv]; 

static int			g_l, g_r, g_p, g_s, avail;
static int			path[maxv];
static int			st[maxe], st2[maxe];
static int			nt0, nt[maxe];
static int			g_f[maxv2];

//static char		OUTFILE[19] = "HopcroftTarjan.out";
//static char		INFILE[11] = "INDATA.dat";

static void		dfs(), palist();
static void		write_illegal(), write_planar(), write_nonplanar();

void	writechr(), graphout(), 
		embed(), add(), pathfind(int), 
		stage1(int), stage2(int,int), stage3(int,int), stage4(int), stage5(int), stage6();
static link   *new_link(), *dlt();
static link2  *new_link2(), *add2(), *dlt2();

static void open_outfile(void)
{
	if ((outfile = fopen("HopcroftTarjan.out", "w")) == NULL) {
		printf("Can't open file : %s\n", "HopcroftTarjan.out");
		retcode = -1;
		longjmp(env, 1);
	}
}

static link *new_link(void)
{
      link  *ptr;

      ptr = (link *)malloc(sizeof(link));
	  listlink[pos++] = ptr;
      if (ptr == NULL) {
           printf("Memory exhausted.\n");
           exit(1);
      }
      else   return(ptr);
}

static link2 *new_link2(void)
{
      link2  *ptr;

      ptr = (link2 *)malloc(sizeof(link2));
	  listlink2[pos2++] = ptr;
      if (ptr == NULL) {
           printf("Memory exhausted.\n");
           exit(1);
      }
      else   
		  return(ptr);
}   
    
static void readgraph(void)
{
	int     vtx, edge, i, j, nu;
	link    *temp, *now;
	int     check[maxv];
	int     st1[maxe], nt1[maxe];

	for (i = 1; i <= maxv; ++i)
		check[i] = 0;
	st1[0] = nt1[0] = nt1[1] = 0;
	g_nv = 1;
	g_ne = 0;    
	fscanf(infile, "%d", &nu);  
	fscanf(infile, "%d", &vtx);  
	fprintf(outfile, "\n %3d: ", vtx);
	orgl[1] = vtx;
	no[vtx] = 0;
	temp = new_link();
	a[vtx] = temp;
	now = temp;
	pa[vtx] = NULL;
	j = 0;
   
	while (fscanf(infile, "%d", &edge) != EOF) {
		if (edge == 0) {
			nt1[g_ne+1] = 0;
			check[vtx] = j;
			fscanf(infile, "%d", &vtx);
			now->value = 0;
			now->next = NULL;
			if(vtx == 0) {
				g_ne = g_ne / 2;
			}
			else {
				++g_nv;
				fprintf(outfile, "\n %3d: ", vtx);
				orgl[g_nv] = vtx;
				no[vtx] = 0;
				temp = new_link();
				a[vtx] = temp;
				now = temp;
				pa[vtx] = NULL;
				j = 0;
			}
		}
		else {
			fprintf(outfile, "%6d", edge);
			if (vtx == edge)     
				write_illegal();
			else {
				++g_ne;
				temp = new_link();
				now->value = edge;
				now->next = temp;
				i = g_ne;
				j = g_ne;
				while (nt1[i] != 0) {
					if (edge == st1[nt1[i]])  
						write_illegal();
					else  
						i = nt1[i];
				}
				st1[g_ne] = edge;
				nt1[g_ne+1] = g_ne;
				now = temp;
			}
		}
	}
	fclose(infile);
	infile = NULL;
}

static void write_illegal(void) 
{
	printf("\n ??? illegal data ??? \n");
	retcode = -1;
	longjmp(env, 1);
}

void add(int tail, int head)
{
	link  *save1;

	if (free1 == NULL) 
		save1 = new_link();
	else {
		save1 = free1;
		free1 = free1->next;
	}
	save1->value = head;
	save1->next = pa[tail];
	pa[tail] = save1;
}

static link2 *dlt2(link2  *dl2)
{
	link2  *save2;

	save2 = dl2->next;
	dl2->next = free2;
	free2 = dl2;
	return(save2);
}

static void palist(int v, int w)
{
	int    j, xx, yy, mx, phi;
	link2  *save2, *save22;
	int    count;

	count = 0;
	if (lpt1[w] > no[v])  
		est = dlt2(est);
	else {
		mx = 2 * no[v];
		while (no[est->lt] >= no[w]) {
			xx = est->lt;
			yy = est->rt;
			if (no[xx] > no[yy]) 
				phi = 2 * no[yy];
			else if (lpt2[yy] < no[xx])
				phi = 2 * lpt1[yy] + 1;
			else
				phi = 2 * lpt1[yy];
			mx = max(mx, phi);
			save2 = est;
			est = save2->next;
			save2->next = bt[phi];
			bt[phi] = save2;
			if (v < w)
				fprintf(outfile, "%6d -%3d(%3d)", xx, yy, phi);
			else
				fprintf(outfile, "%6d -%3d(%3d)", xx, yy, g_nv+1);
			++count;
			if (count % 5 == 0)   
				fprintf(outfile, "\n");
	    }
		phi = 2 * lpt1[w];
		save2 = est;
		est = save2->next;
		save2->next = bt[phi];
		bt[phi] = save2;
		for (j = mx; j >= 2 * no[v]; --j) {
			if (bt[j] != NULL) {
				save2 = bt[j];
				save22 = save2->next;
				add(save2->lt, save2->rt);
				while (save22 != NULL) {
					save2 = save22;
					save22 = save22->next;
					add(save2->lt, save2->rt);
				}
				save2->next = free2;
				free2 = bt[j];
				bt[j] = NULL;
			}
		}
	}
	if (v < w)
		fprintf(outfile, "%6d -%3d(%3d)\n", v, w, phi);
	else
		fprintf(outfile, "%6d -%3d(%3d)\n", v, w, g_nv+1);
}

link2 *add2(int lft, int rgt, link2  *be)
{
	link2   *save2;

	if (free2 == NULL)
		save2 = new_link2();
	else {
		save2 = free2;
		free2 = free2->next;
	}
	save2->lt = lft;
	save2->rt = rgt;
	save2->next = be;
	return (save2);
}

link *dlt(link  *dl)
{
	link  *save1;

	save1 = dl->next;
	dl->next = free1;
	free1 = dl;
	return(save1);
}

void embed(int start)
{
	g_p = 0;
	g_s = 0;  
	st[0] = 0;
	nt0 = 0;
	nt[0] = 0;
	avail = 1;  
	path[start] = 1; 
	pathfind(start);
}

void pathfind(int x)
{
	int   y;

	while (pa[x] != NULL) {
		y = pa[x]->value;
		if (g_s == 0) {
			g_s = x;
			++g_p;
			fprintf(outfile, "\n no.(%3d) path: %3d -", g_p, x);
		}
		if (no[x] < no[y]) {
			fprintf(outfile, "%3d -", y);
			path[y] = g_p; 
			pathfind(y);
			stage1(x);
			if (path[x] != path[y])
				stage2(x, y); 
	    }
		else {
			fprintf(outfile, "%3d", y);
			g_f[g_p] = no[y];
			stage3(x, y); 
			if (g_p == 1)
				fprintf(outfile, "\n Please embed no.( 1) path counterclockwisely.\n");
			else {
				fprintf(outfile, "\n Please embed no.(%3d) path on the inside", g_p);
				fprintf(outfile, " of the corresponding cycle.\n");
				if (g_f[path[g_s]] < no[y])
					stage4(y); 
			}
			stage5(x); 
			if (x != g_s) 
				stage6(); 
			g_s = 0;
	    }
		pa[x] = dlt(pa[x]);
	}                        
}
 
void stage1(int x)
{
	while (((st[bst->lt] >= no[x]) || (bst->lt == 0)) && \
             ((st[bst->rt] >= no[x]) || (bst->rt == 0)) && (bst->next != NULL))          
		bst = dlt2(bst);
	if (st[bst->lt] >= no[x])          
		bst->lt = 0;
	if (st[bst->rt] >= no[x])          
		bst->rt = 0;
	while ((nt0 != 0) && (st[nt0] >= no[x]))     
		nt0 = nt[nt0];            
	while ((nt[0] != 0) && (st[nt[0]] >= no[x]))       
		nt[0] = nt[nt[0]];
}

void stage2(int x, int y)
{
	int  save;

	g_l = 0;
	while ((st[bst->lt] > g_f[path[y]]) || (st[bst->rt] > g_f[path[y]]) && \
              (st[nt0] != 0))  {
		if (st[bst->lt] > g_f[path[y]]) {
			if (st[bst->rt] > g_f[path[y]]) {
				fprintf(outfile, "\n @ the segment whose first path is no.(%3d)", path[y]);
				fprintf(outfile, " can not be embedded in the plane as above.\n");
				write_nonplanar();
			}
			g_l = bst->lt;
		}
		else {
			fprintf(outfile, "\n Please move no.(%3d", st2[bst->rt]);
			if (bst->rt == nt0) 
				fprintf(outfile, ") path");
			else       
				fprintf(outfile, ") - no.(%3d) paths", st2[nt0]);
			fprintf(outfile, " on the outside to inside, and if necessary, move some");
			fprintf(outfile, " paths above the vertex(%3d)\n", x);
			save = nt[g_l];
			nt[g_l] = nt0;
			nt0 = nt[bst->rt];
			nt[bst->rt] = save;
			g_l = bst->rt;
	    }
		bst = dlt2(bst);
	}
	fprintf(outfile, "\n @ the segment whose first path is no.(%3d)", path[y]);
	fprintf(outfile, " can be embedded on the inside as above.\n");
	if (bst->lt == 0) {
		if ((g_l == 0) && (bst->rt == 0)) 
			bst = dlt2(bst);
		else       
			bst->lt = g_l;
	}
	nt0 = nt[nt0];
}
 
void stage3(int x, int y)
{     
	int  save;
      
	g_l = 0;
	g_r = -1;
	while ((nt[g_l] != 0 && st[nt[g_l]] > no[y]) || 
		(g_r >= 0 && nt[g_r] != 0 && st[nt[g_r]] > no[y]) ||
		(g_r == -1 && nt0 != 0 && st[nt0] > no[y])) {
		if (bst->lt == 0) {
			if (bst->rt != 0)
				g_r = bst->rt;
		}
		else if (bst->rt == 0) {
			fprintf(outfile, "\n Please move no.(%3d", st2[bst->lt]);
			if (bst->lt == nt[g_l])
				fprintf(outfile, ") path");
			else 
				fprintf(outfile, ") - no.(%3d) paths", st2[nt[g_l]]);
			fprintf(outfile, " on the inside and to outside, and if necessary,"); 
			fprintf(outfile, " move some path above the vertex(%3d)\n", x);
			save = nt[bst->lt];
			if (g_r == -1) {
				nt[bst->lt] = nt0;
				nt0 = nt[g_l];
			}
			else {
				nt[bst->lt] = nt[g_r];
				nt[g_r] = nt[g_l];
			}
			nt[g_l] = save;
			g_r = bst->lt;
		}
		else if (st[nt[g_l]] > no[y]) {
			if (g_r >= 0 && st[nt[g_r]] > no[y] || st[nt0] > no[y]) {
				fprintf(outfile, "\n @ no.(%3d) path can not be embedded\n", g_p);
				write_nonplanar();
			}
			fprintf(outfile, "\n Please interchange no.(%3d", st2[bst->lt]);
			if (bst->lt == nt[g_l])
				fprintf(outfile, ") path");
			else   
				fprintf(outfile, ") - no.(%3d) paths", st2[nt[g_l]]);
			fprintf(outfile, " on the inside and no.(%3d", st2[nt[g_l]]);
			if (g_r >= 0 && bst->rt == nt[g_r] || bst->rt == nt0)
				fprintf(outfile, ") path");
			else {
				if (g_r == -1)
					fprintf(outfile, ") - no.(%3d) paths", st2[nt0]);
				else
					fprintf(outfile, ") - no.(%3d) paths", st2[nt[g_r]]);
			}
			fprintf(outfile, " on the outside, and if necessary, move some paths");
			fprintf(outfile, " above the vertex(%3d)\n", x);
			if (g_r == -1) {
				save = nt0;
				nt0 = nt[g_l];
			}
			else {
				save = nt[g_r];
				nt[g_r] = nt[g_l];
			}
			nt[g_l] = save;
			save = nt[bst->lt];
			nt[bst->lt] = nt[bst->rt];
			nt[bst->rt] = save;
			g_l = bst->rt;
			g_r = bst->lt;
		}
		else {
			g_l = bst->lt;
			g_r = bst->rt;
		}
		bst = dlt2(bst);
	}
}
 
void stage4(int y)
{
	if (g_l == 0)    
		g_l = avail;
	st[avail] = no[y];
	st2[avail] = g_p;
	nt[avail] = nt[0];
	nt[0] = avail;
	++avail;
}

void stage5(int x)
{  
	if (g_r == -1)
		g_r = 0;
	if ((g_l != 0) || (g_r != 0) || (x != 5))
		bst = add2(g_l, g_r, bst);
}

void stage6(void)
{     
	st[avail] = 0;
	nt[avail] = nt0;
	nt0 = avail;
	++avail;
}

static void dfs(int v, int u)
{
	int   w;

	g_k++;
	no[v] = lpt1[v] = lpt2[v] = g_k;
	while (a[v]->next != NULL) {
		w = a[v]->value;
		if (no[w] == 0) {
			est = add2(v, w, est);
			dfs(w, v);
			if (lpt1[w] < lpt1[v]) { 
				lpt2[v] = min(lpt1[v], lpt2[w]);
				lpt1[v] = lpt1[w];
			}
			else if (lpt1[w] == lpt1[v]) 
				lpt2[v] = min(lpt2[v], lpt2[w]);
			else
				lpt2[v] = min(lpt2[v], lpt1[w]);
			if (lpt1[w] >= no[v]) {
				++g_ct;
				fprintf(outfile, "\n ### no.(%3d) bicomponent (v, w(phi)) ###\n", g_ct);
				palist(v, w);
				embed(v);
				fprintf(outfile, "\n\n --> this bicomponent is planar \n\n");
				if (pa[v] != NULL)
					pa[v] = dlt(pa[v]);
			}
		}
		else if ((w != u) && (no[w] < no[v])) {
			est = add2(v, w, est);
			if (no[w] < lpt1[v]) {
				lpt2[v] = lpt1[v];
				lpt1[v] = no[w];
			}
			else if (no[w] > lpt1[v]) 
				lpt2[v] = min(lpt2[v], no[w]);
		}
		a[v] = dlt(a[v]);
	}
}

static void write_planar(void)
{
	fprintf(outfile, "\n *** This graph is planar by Hopcroft-Tarjan algorithm *** \n");
	retcode = 1;
	longjmp(env, 1);
}

static void write_nonplanar(void)
{
	fprintf(outfile, "\n *** This graph is nonplanar by Hopcroft-Tarjan algorithm *** \n");
	retcode = 0;
	longjmp(env, 1);
}

int HopcroftTarjan(void)
{
	int		i;

	pos = 0; pos2 = 0;
	if (setjmp(env)) {
		for (i = 0; i < pos; ++i)
			free((char*)listlink[i]);
		for (i = 0; i < pos2; ++i)
			free((char*)listlink2[i]);
//		printf("(%d, %d)\n", pos, pos2);
		if (outfile != NULL)
			fclose(outfile);
		fflush(stdout);
		return(retcode);
	}

	if (infile != NULL) {
		; /* already opened */
	}
	else if ((infile = fopen("INDATA.dat", "r")) == NULL)  {
		printf("Can't open file : %s\n", "INDATA.dat");
		retcode = -1;
		return(retcode);
	}

	outfile = NULL;
	open_outfile();
	if (outfile == NULL) {
		retcode = -1;
		longjmp(env, 1);
	}
	if (INDATA == 0)
		fprintf(outfile, "*** Testing for the data file %s *** \n", filename);

	readgraph();
                
	free1 = NULL; 
	free2 = est = NULL;
	bst = new_link2();
	bst->next = NULL;
	bst->lt = 0;
	bst->rt = 0;

	fprintf(outfile, "\n @@@ adjacency list @@@ \n\n");
	fprintf(outfile, "\n");

	if (g_nv < 5 || g_ne < 9) {      
		write_planar(); 
		return(1);
	}
	if ((g_ne > 3*g_nv-3) || (g_nv >= 3 && g_ne > 3*g_nv-6)) {
		write_nonplanar();
		return(0);
	}
	for (i = 0; i <= 2*g_nv+1; ++i)    
		bt[i] = NULL;
	g_k = 0;
	g_ct = 0;
	for (i = 1; i <= g_nv; ++i) {
		if (no[orgl[i]] == 0) {
			if (a[orgl[i]]->next == NULL) {
				++g_ct;
				fprintf(outfile, "\n ### no.(%3d) bicomponent ### \n\n", g_ct);
				fprintf(outfile, "%d (isolated)\n\n", orgl[i]);
				fprintf(outfile, "--> this component is planar \n\n");
		    }
			else  
				dfs(orgl[i], 0);
		}
	} 
	write_planar();
	return(1);
}






