#include	<stdio.h>
#include	<stdlib.h>

#define 	MAXSIZE 	100

typedef struct fnode {
	int setname,next;
} FIND;
typedef struct unode {
	int list,size;
} UNION;
typedef struct gnode {
	int v1,v2,cost;
} GRAPH;
typedef int ARRAY[MAXSIZE];

void exchange(int *i, int *j);
void buildheap(int n, int *index,int *key);
void heapsort(int n, int *index, int *key);
void union_find(int n, FIND *F, UNION *U, GRAPH *G);

void main(void)
{
	int 	i,k,n,v,e,edge,pos;
	GRAPH 	*P,G[MAXSIZE];
	FIND  	F[MAXSIZE];
	UNION 	U[MAXSIZE];
	ARRAY 	index,key,tree;
	char 	graph_f[13];
	FILE 	*fopen(),*fp_in;
	
	printf("graph data file ? ");
	scanf("%s",graph_f);
	if ((fp_in = fopen(graph_f,"r")) == NULL) {
		printf("Can't open a file %s\n", graph_f);
		exit(0);
	}
	fscanf(fp_in,"%d %d",&v,&e);
	n = 0;
	while (n < e) {
		n++;
		P = G+n;
		fscanf(fp_in,"%d %d %d",&P->v1,&P->v2,&P->cost);
		key[n] = G[n].cost;
		index[n] = n;
	}
	heapsort(n,index,key); /* n is number of edges */
	for (i = 1; i <= v; i++) {
		F[i].setname = i;
		F[i].next = 0;
		U[i].list = i;
		U[i].size = 1;
	}
	k = 1;
	edge = index[k];
	pos = 1;
	while (pos < v) {
	    if (!(F[G[edge].v1].setname == F[G[edge].v2].setname)) {
		union_find(edge,F,U,G);
		tree[pos++] = edge;
	    }
	    edge = index[++k];
	}
	printf("The list of an optimal tree whose cost is minimal:\n");
	for (i = 1; i < v; i++)
		printf("(%5d,%5d)\n",G[tree[i]].v1,G[tree[i]].v2);
	fclose(fp_in);
}

void exchange(int *i, int *j)
{
	int temp;
	
	temp = *i; *i = *j; *j = temp;
}

void heapify(int i, int j, int *index, int *key)
{
	
	if (i*2+1 <= j)  /* 'i' has both left and right son */
		if (key[index[2*i]] > key[index[2*i+1]]) {
			if (key[index[2*i]] > key[index[i]]) {
				exchange(index+(2*i),index+i);
				heapify(2*i,j,index,key);
			}
		}
		else {
			if (key[index[2*i+1]] > key[index[i]]) {
				exchange(index+(2*i)+1,index+i);
				heapify(2*i+1,j,index,key);
			}
		}
	else if (2*i == j) /* 'i' has only left son */
		if (key[index[2*i]] > key[index[i]]) {
			exchange(index+(2*i),index+i);
			heapify(2*i,j,index,key);
		}
}

void buildheap(int n, int *index,int *key)
{
	register int i;
	
	for (i = n/2; i >= 1; i--)
		heapify(i,n,index,key);
}

void heapsort(int n, int *index, int *key)
{
	register int i;
	
	buildheap(n,index,key);
	for (i = n; i >= 2; i--) {
		exchange(index+1,index+i);
		heapify(1,i-1,index,key);
	}
}

void union_find(int n, FIND *F, UNION *U, GRAPH *G)
{
	int set1,set2,element,last;
	
	set1 = F[G[n].v1].setname;/* set1 = (F+(G+n)->v1)->setname */
	set2 = F[G[n].v2].setname;/* set2 = (F+(G+n)->v2)->setname */
	if (U[set1].size > U[set2].size)
		exchange(&set1,&set2);
	element = U[set1].list;
	while (element) {
		F[element].setname = set2;
		last = element;
		element = F[element].next;
	}
	F[last].next = U[set2].list;
	U[set2].list = U[set1].list;
	U[set2].size = U[set1].size + U[set2].size;
	
}

/* a graph with 7 vertexes, 14 edges: 1 7 1 edge (1,7) has a wight 1
7 14
1 7 1
3 4 3
2 4 11
2 5 2
3 7 9
2 3 15
4 7 16
4 5 17
1 2 20
1 3 8
1 6 23
5 7 25
5 6 28
6 7 36

graph data file ? graph.dat
Edge list of maximal tree whose cost is minimal:
(    1,    7)
(    2,    5)
(    3,    4)
(    1,    3)
(    2,    4)
(    1,    6)
*/