// draw.cpp   1998 by Masaaki Wada. All rights reserved.
// 19980521 Masaaki Wada
// 19980610MW CWPro3
// 19980614MW name change mDraw -> mIDraw
#include "stdafx.h" //MFC ̂

#include	"draw.h"


// ================================================================================
// draw

draw::draw( iDraw& d,
			double xmin, double xmax,
			double ymin, double ymax,
			int keep_ratio )
	: mIDraw( d )
{
	mKeepRatio = keep_ratio;
	SetRange( xmin, xmax, ymin, ymax );
}


void
draw::Adjust()
{
	mXPixel = ( mXMax - mXMin ) / ( mIDraw.Right() - mIDraw.Left() );
	mYPixel = ( mYMax - mYMin ) / ( mIDraw.Bottom() - mIDraw.Top() );
	if ( mXPixel > mYPixel ) {
		if ( mKeepRatio )
			mYPixel = mXPixel;
		mPixel = mYPixel;
	} else {
		if ( mKeepRatio )
			mXPixel = mYPixel;
		mPixel = mXPixel;
	}
	mXOrigin = 0.5 * ( mXMin + mXMax - ( mIDraw.Left() + mIDraw.Right() ) * mXPixel );
	mYOrigin = 0.5 * ( mYMin + mYMax + ( mIDraw.Top() + mIDraw.Bottom() ) * mYPixel );
}


// --------------------------------------------------------------------------------

void
draw::DrawLine(  pvect v0, pvect v1, double width, rgb color )
{
	if ( width <= 1.0 * mPixel )
		mIDraw.DrawLine( Ptrunc( v0 ), Ptrunc( v1 ), color );
	else {
		pvect	e = ~( v1 - v0 ).normalized() * ( 0.5 * width );
		pvect	vertex[ 4 ] = { v0 + e, v0 - e, v1 - e, v1 + e };
		PaintPolygon( vertex, 4, color );
	}
}


void
draw::DrawBezier( pvect v0, pvect v1, pvect v2, pvect v3, double width, rgb color )
{
	const double	tolerance = 0.5;
	pvect	w03 = v3 - v0;
	double	norm03 = norm( w03 );
	pvect	e03 = w03.normalized();
	pvect	w01 = v1 - v0;
	double	ip1 = w01 | w03;
	pvect	w23 = v3 - v2;
	double	ip2 = w23 | w03;
	if ( ( fabs( w01 ^ e03 ) < tolerance * mPixel ) &&
			( fabs( w23 ^ e03 ) < tolerance * mPixel ) &&
			( ip1 >= 0 ) && ( ip2 >=0 ) && ( ip1 + ip2 <= norm03 ) ) {
		if ( width <= 1.5 * mPixel )
			DrawLine( v0, v3, width, color );
		else {
			pvect	e0 = ~w01.normalized() * ( 0.5 * width );
			pvect	e3 = ~w23.normalized() * ( 0.5 * width );
			pvect	vertex[ 4 ] = { v0 + e0, v0 - e0, v3 - e3, v3 + e3 };
			PaintPolygon( vertex, 4, color );
			}
		}
	else {
		pvect	v01 = 0.5 * ( v0 + v1 );
		pvect	v12 = 0.5 * ( v1 + v2 );
		pvect	v23 = 0.5 * ( v2 + v3 );
		pvect	v012 = 0.5 * ( v01 + v12 );
		pvect	v123 = 0.5 * ( v12 + v23 );
		pvect	v0123 = 0.5 * ( v012 + v123 );
		DrawBezier( v0, v01, v012, v0123, width, color );
		DrawBezier( v0123, v123, v23, v3, width, color );
		}
}


void
draw::DrawArc( pvect center, double radius, int start, int sweep, double width, rgb color )
{
	radius += 0.5 * width;
	int			diam = (int)( 2.0 * radius / mPixel + 0.5 );
	if ( diam < 1 ) diam = 1;
	int			i, j;
	if ( diam % 2 ) {
		i = Itrunc( center.x ) - diam / 2;
		j = Jtrunc( center.y ) - diam / 2;
	} else {
		i = I( center.x ) - diam / 2;
		j = J( center.y ) - diam / 2;
	}
	iRect		r( i, j, i + diam, j + diam );
	int		w = (int)( width / mPixel + 0.5 );
	if ( w < 1 ) w = 1;
	mIDraw.DrawOval( r, start, sweep, w, color );
}


void
draw::PaintWedge( pvect center, double radius, int start, int sweep,  rgb color )
{
	int			diam = (int)( 2.0 * radius / mPixel + 0.5 );
	if ( diam < 1 ) diam = 1;
	int			i, j;
	if ( diam % 2 ) {
		i = Itrunc( center.x ) - diam / 2;
		j = Jtrunc( center.y ) - diam / 2;
	} else {
		i = I( center.x ) - diam / 2;
		j = J( center.y ) - diam / 2;
	}
	iRect		r( i, j, i + diam, j + diam );
	mIDraw.PaintOval( r, start, sweep, color );
}


void
draw::DrawPolygon( pvect vertex[], int num_vertices, rgb color )
{
	iPoint*		pv = new iPoint[ num_vertices ];
	if ( pv != 0 ) {
		for ( int i = 0 ; i < num_vertices ; i++ )
			pv[ i ] = Ptrunc( vertex[ i ] );
		mIDraw.DrawPolygon( pv, num_vertices, color );
		delete pv;
	}
}


void
draw::PaintPolygon( pvect vertex[], int num_vertices, rgb color )
{
	iPoint*		pv = new iPoint[ num_vertices ];
	if ( pv != 0 ) {
		for ( int i = 0 ; i < num_vertices ; i++ )
			pv[ i ] = P( vertex[ i ] );
		mIDraw.PaintPolygon( pv, num_vertices, color );
		delete pv;
	}
}


void
draw::DrawString( pvect v, string str, int font_size, string font_name,
														int centering, rgb color )
{
	mIDraw.DrawString( P( v ), str, font_size, font_name, centering, color );
}


void
draw::DrawString( pvect v, string str, double font_sz, string font_name,
														int centering, rgb color )
{
	int			font_size;
	if ( font_sz < YPixel() )
		font_size = 1;
	else if ( font_sz > 1000.0 * YPixel() )
		font_size = 1000;
	else
		font_size = (int)( font_sz / YPixel() + 0.5 );
	mIDraw.DrawString( P( v ), str, font_size, font_name, centering, color );
}



// ================================================================================
// shapes

// --------------------------------------------------------------------------------
// polygon

polygon::polygon( rgb color, int initial_size )
	: shape( color )
{
	mNumVertices = 0;
	mArraySize = initial_size;
	mVertex = new pvect[ initial_size ];
}


polygon::polygon( const polygon& p )
	: shape( p.mColor, p.mBase )
{
	mNumVertices = p.mNumVertices;
	mArraySize = p.mArraySize;
	mVertex = new pvect[ mArraySize ];
	for ( int i = 0 ; i < mNumVertices ; i++ )
		mVertex[ i ] = p.mVertex[ i ];
}


polygon::~polygon()
{
	delete mVertex;
}


void
polygon::AddVertex( pvect v )
{
	if ( mNumVertices >= mArraySize ) {
		pvect*		tmp = mVertex;
		mArraySize += ARRAY_INCREMENT_SIZE;
		mVertex = new pvect[ mArraySize ];
		for ( int i = 0 ; i < mNumVertices ; i++ )
			mVertex[ i ] = tmp[ i ];
		delete tmp;
	}
	mVertex[ mNumVertices++ ] = v;
}


void
polygon::Draw( draw& d )
{
	pvect*		tmp = new pvect[ mNumVertices ];
	for ( int i = 0 ; i < mNumVertices ; i++ )
		tmp[ i ] = mVertex[ i ] + mBase;
	d.PaintPolygon( tmp, mNumVertices, mColor );
	delete tmp;
}


// --------------------------------------------------------------------------------
// pl_curve

void
pl_curve::Draw( draw& d )
{
	pvect*		tmp = new pvect[ mNumVertices ];
	for ( int i = 0 ; i < mNumVertices ; i++ )
		tmp[ i ] = mVertex[ i ] + mBase;
	d.DrawPolygon( tmp, mNumVertices, mColor );
	delete tmp;
}


// --------------------------------------------------------------------------------
// textstring

textstring::textstring( pvect loc,
						string str,
						int font_size,
						string font_name,
						int centering,
						rgb color )
	: shape( color, loc )
{
	mString = str;
	mFontName = font_name;
	mFontSize = font_size;
	mFixedSize = true;
	mCentering = centering;
}


textstring::textstring( double x,
						double y,
						string str,
						int font_size,
						string font_name,
						int centering,
						rgb color )
	: shape( color, x, y )
{
	mString = str;
	mFontName = font_name;
	mFontSize = font_size;
	mFixedSize = true;
	mCentering = centering;
}


textstring::textstring( pvect loc,
						string str,
						double font_sz,
						string font_name,
						int centering,
						rgb color )
	: shape( color, loc )
{
	mString = str;
	mFontName = font_name;
	mFontSz = font_sz;
	mFixedSize = false;
	mCentering = centering;
}


textstring::textstring( double x,
						double y,
						string str,
						double font_sz,
						string font_name,
						int centering,
						rgb color )
	: shape( color, x, y )
{
	mString = str;
	mFontName = font_name;
	mFontSz = font_sz;
	mFixedSize = false;
	mCentering = centering;
}


textstring::textstring( textstring& s )
	: shape( s.mColor, s.mBase )
{
	mString = s.mString;
	mFontName = s.mFontName;
	mFontSize = s.mFontSize;
	mFixedSize = s.mFixedSize;
	mCentering = s.mCentering;
}


void
textstring::Draw( draw& d )
{
	if ( mFixedSize )
		d.DrawString( mBase, mString, mFontSize, mFontName, mCentering, mColor );
	else
		d.DrawString( mBase, mString, mFontSz, mFontName, mCentering, mColor );
}



// --------------------------------------------------------------------------------
// composite : collection of shapes

composite::composite()
	: shape()
{
	mNumShapes = 0;
	mArraySize = ARRAY_INCREMENT_SIZE;
	mShape = new shape*[ mArraySize ];
}


composite::composite( const composite& s )
	: shape( s.mColor, s.mBase )
{
	mNumShapes = s.mNumShapes;
	mArraySize = s.mArraySize;
	mShape = new shape*[ mArraySize ];
	for ( int i = 0 ; i < mNumShapes ; i++ )
		mShape[ i ] = ( s.mShape[ i ] )->Clone();
}


composite::~composite()
{
	Clear();
	delete mShape;
}


void
composite::Clear( void )
{
	for ( int i = 0 ; i < mNumShapes ; i++ )
		delete mShape[ i ];
	mNumShapes = 0;
}


void
composite::AddShape( shape* p )
{
	if ( mNumShapes >= mArraySize ) {
		shape**			tmp = mShape;
		mArraySize += ARRAY_INCREMENT_SIZE;
		mShape = new shape*[ mArraySize ];
		for ( int i = 0 ; i < mNumShapes ; i++ )
			mShape[ i ] = tmp[ i ];
		delete tmp;
	}
	mShape[ mNumShapes++ ] = p;
}


void
composite::SetColor( rgb color )
{
	for ( int i = 0 ; i < mNumShapes ; i++ )
		mShape[ i ]->SetColor( color );
}


void
composite::Draw( draw& d )
{
	for ( int i = 0 ; i < mNumShapes ; i++ ) {
		shape*		s = mShape[ i ]->Clone();
		s->Translate( mBase );
		s->Draw( d );
		delete s;
	}
}


// end of draw.cpp
