// iDraw.cpp   1998 by Masaaki Wada. All rights reserved.
// 19980520 Masaaki Wada
// 19980610MW CWPro3
#include "stdafx.h" //MFC only
#include	"iDraw.h"
#include	<math.h>


// ================================================================================
//                               rgb
// ================================================================================


rgb::rgb( scale_t scale, double m, double mmin, double mmax )
{
	if ( scale == color_scale )
		ColorScale( m, mmin, mmax );
	else
		GrayScale( m, mmin, mmax );
}


rgb::rgb( scale_t scale, int k, int kmin, int kmax )
{
	if ( scale == color_scale )
		ColorScale( k, kmin, kmax );
	else
		GrayScale( k, kmin, kmax );
}


void
rgb::Set( double r, double g, double b )
{
	if ( r < 0 )
		red = 0.0;
	else if ( r > 1.0 )
		red = 1.0;
	else
		red = r;
	if ( g < 0 )
		green = 0.0;
	else if ( g > 1.0 )
		green = 1.0;
	else
		green = g;
	if ( b < 0 )
		blue = 0.0;
	else if ( b > 1.0 )
		blue = 1.0;
	else
		blue = b;
}


void
rgb::ColorScale( double m, double mmin, double mmax )
{
	double		t;

	if ( mmin < mmax )
		m = ( m - mmin ) / ( mmax - mmin );
	else
		m = 0.0;

	if ( m < 0.4 ) {
		if ( m < 0.0 ) {			//        m < 0.0
			t = exp( 5.0 * m );
			Set( 0.0, 0.0, t );
			}
		else if ( m < 0.2 ) {		// 0.0 <= m < 0.2
			t = m / 0.2;
			Set( 0.0, 0.75 * t, 1.0 - 0.25 * t );
			}
		else {						// 0.2 <= m < 0.4
			t = ( m - 0.2 ) / 0.2;
			Set( 0.0, 0.75 + 0.25 * t, 0.75 - 0.75 * t );
			}
		}
	else /* if ( m >= 0.4 ) */ {
		if ( m < 0.8 ) {
			if ( m < 0.6 ) {		// 0.4 <= m < 0.6
				t = ( m - 0.4 ) / 0.2;
				Set( 0.75 * t, 1.0 - 0.25 * t, 0.0 );
				}
			else {					// 0.6 <= m < 0.8
				t = ( m - 0.6 ) / 0.2;
				Set( 0.75 + 0.25 * t, 0.75 - 0.75 * t, 0.0 );
				}
			}
		else {
			if ( m < 1.0 ) {		// 0.8 <= m < 1.0
				t = ( m - 0.8 ) / 0.2;
				Set( 1.0, 0.0, t );
				}
			else {					// 1.0 <= m
				m -= 1.0;
				t = 1.0 - exp( -5.0 * m );
				Set( 1.0, t, 1.0 );
				}
			}
		}
}


void
rgb::ColorScale( int k, int kmin, int kmax )
{
	ColorScale( (double) k, (double) kmin, (double) kmax );
}


void
rgb::GrayScale( double m, double mmin, double mmax )
{
	if ( mmin < mmax )
		m = ( m - mmin ) / ( mmax - mmin );
	else
		m = 0.0;

	Set( m, m, m );
}


void
rgb::GrayScale( int k, int kmin, int kmax )
{
	GrayScale( (double) k, (double) kmin, (double) kmax );
}


// ================================================================================
//                               shapes
// ================================================================================


// --------------------------------------------------------------------------------
// iPolyFrame
iPolyFrame::iPolyFrame( rgb color, int initial_size )
	: iShape( color )
{
	mNumVertices = 0;
	mArraySize = initial_size;
	mVertex = new iPoint[ initial_size ];
}


iPolyFrame::iPolyFrame( const iPolyFrame& poly )
	: iShape( poly.mColor, poly.mBase )
{
	mNumVertices = poly.mNumVertices;
	mArraySize = poly.mArraySize;
	mVertex = new iPoint[ mArraySize ];
	for ( int i = 0 ; i < mNumVertices ; i++ )
		mVertex[ i ] = poly.mVertex[ i ];
}


iPolyFrame::~iPolyFrame()
{
	delete mVertex;
}


void
iPolyFrame::AddVertex( iPoint p )
{
	if ( mNumVertices >= mArraySize ) {
		iPoint*		tmp = mVertex;
		mArraySize += ARRAY_INCREMENT_SIZE;
		mVertex = new iPoint[ mArraySize ];
		for ( int i = 0 ; i < mNumVertices ; i++ )
			mVertex[ i ] = tmp[ i ];
		delete tmp;
	}
	mVertex[ mNumVertices++ ] = p;
}


void
iPolyFrame::Draw( iDraw& d )
{
	iPoint*		tmp = new iPoint[ mNumVertices ];
	for ( int i = 0 ; i < mNumVertices ; i++ )
		tmp[ i ] = mVertex[ i ] + mBase;
	d.DrawPolygon( tmp, mNumVertices, mColor );
	delete tmp;
}


// --------------------------------------------------------------------------------
// iPolygon
iPolygon::iPolygon( rgb color, int initial_size )
	: iShape( color )
{
	mNumVertices = 0;
	mArraySize = initial_size;
	mVertex = new iPoint[ initial_size ];
}


iPolygon::iPolygon( const iPolygon& poly )
	: iShape( poly.mColor, poly.mBase )
{
	mNumVertices = poly.mNumVertices;
	mArraySize = poly.mArraySize;
	mVertex = new iPoint[ mArraySize ];
	for ( int i = 0 ; i < mNumVertices ; i++ )
		mVertex[ i ] = poly.mVertex[ i ];
}


iPolygon::~iPolygon()
{
	delete mVertex;
}


void
iPolygon::AddVertex( iPoint p )
{
	if ( mNumVertices >= mArraySize ) {
		iPoint*		tmp = mVertex;
		mArraySize += ARRAY_INCREMENT_SIZE;
		mVertex = new iPoint[ mArraySize ];
		for ( int i = 0 ; i < mNumVertices ; i++ )
			mVertex[ i ] = tmp[ i ];
		delete tmp;
	}
	mVertex[ mNumVertices++ ] = p;
}


void
iPolygon::Draw( iDraw& d )
{
	iPoint*		tmp = new iPoint[ mNumVertices ];
	for ( int i = 0 ; i < mNumVertices ; i++ )
		tmp[ i ] = mVertex[ i ] + mBase;
	d.PaintPolygon( tmp, mNumVertices, mColor );
	delete tmp;
}


// --------------------------------------------------------------------------------
// iComposite
iComposite::iComposite()
	: iShape()
{
	mNumShapes = 0;
	mArraySize = ARRAY_INCREMENT_SIZE;
	mShape = new iShape*[ ARRAY_INCREMENT_SIZE ];
}


iComposite::iComposite( const iComposite& s )
{
	mNumShapes = s.mNumShapes;
	mArraySize = s.mArraySize;
	mShape = new iShape*[ mArraySize ];
	for ( int i = 0 ; i < mNumShapes ; i++ )
		mShape[ i ] = ( s.mShape[ i ] )->Clone();
}


iComposite::~iComposite()
{
	for ( int i = 0 ; i < mNumShapes ; i++ )
		delete mShape[ i ];
	delete mShape;
}


void
iComposite::Clear( void )
{
	for ( int i = 0 ; i < mNumShapes ; i++ )
		delete mShape[ i ];
	mNumShapes = 0;
}


void
iComposite::AddShape( iShape* p )
{
	if ( mNumShapes >= mArraySize ) {
		iShape**		tmp = mShape;
		mArraySize += ARRAY_INCREMENT_SIZE;
		mShape = new iShape*[ mArraySize ];
		for ( int i = 0 ; i < mNumShapes ; i++ )
			mShape[ i ] = tmp[ i ];
		delete tmp;
	}
	mShape[ mNumShapes++ ] = p;
}


void
iComposite::SetColor( rgb color )
{
	for ( int i = 0 ; i < mNumShapes ; i++ )
		mShape[ i ]->SetColor( color );
}


void
iComposite::Draw( iDraw& d )
{
	for ( int i = 0 ; i < mNumShapes ; i++ ) {
		iShape*		s = mShape[ i ]->Clone();
		s->Translate( mBase );
		s->Draw( d );
		delete s;
	}
}




// end of iDraw.cpp
