//*****************************************//
// Ochiai Lab. Nara Women's University     //
// Programmed by Chihiro Taninaka          //
// All rights reserved. March 1999         //
//*****************************************//
#include "stdafx.h"
#include "mfcDraw.h"
#include <math.h>

#define SAVEPEN			CPen    myPen, *pOldPen; \
						myPen.CreatePen( PS_SOLID, 1, GetColor(color) ); \
						pOldPen   = pDC->SelectObject(&myPen);

#define SAVEBRUSH		CBrush  myBrush, *pOldBrush; \
	                    myBrush.CreateSolidBrush(GetColor(color)); \
				        pOldBrush = pDC->SelectObject(&myBrush); 

#define RESTOREPEN		pDC->SelectObject(pOldPen);\
						myPen.DeleteObject();

#define RESTOREBRUSH    pDC->SelectObject(pOldBrush);\
					    myBrush.DeleteObject(); 

//------------------------------------------------------------------------
// mfcDraw

//------------------------------------------------------------------------
// overriding funcions

void mfcDraw::IDrawLine(int i0, int j0, int i1, int j1, rgb color)
{
	if( (i0 != i1) || (j0 != j1) ) {
		SAVEPEN
		pDC->MoveTo(i0,j0);
		pDC->LineTo(i1,j1);
		RESTOREPEN;
	}
}


void
mfcDraw::IDrawRectangle( int l, int t, int r, int b, int width, rgb color )
{
	if ( ( l < r ) && ( t < b ) ) {
		RECT	rct = { l, t, r, b };
		SAVEBRUSH;
		pDC->FrameRect( &rct, &myBrush );
		RESTOREBRUSH;
	}
}


void
mfcDraw::IPaintRectangle( int l, int t, int r, int b, rgb color )
{
	if ( ( l < r ) && ( t < b ) ) {
		RECT	rct = { l, t, r, b };
		SAVEBRUSH;
		pDC->Rectangle( &rct );
		RESTOREBRUSH;
	}
}


void
mfcDraw::IDrawOval( int l, int t, int r, int b, int start, int sweep, int width, rgb color )
{
	if ( ( l < r ) && ( t < b ) ) {
		RECT	rct = { l, t, r, b };
		POINT   pntu, pntv;

		CPen  myPen;         // Pen ɕ邽߁A
		CPen* pOldPen;       // SAVEPEN ͎gȂ
		myPen.CreatePen( PS_INSIDEFRAME, width, GetColor(color) );
		pOldPen = pDC->SelectObject(&myPen);

		double     st = 2 * pi * start / 360;
		double     sw = 2 * pi * (start + sweep) / 360;

		pntu.x = (l + r) / 2 + (r - l) / 2 * (cos( st ));
		pntu.y = (t + b) / 2 - (b - t) / 2 * (sin( st ));
		pntv.x = (l + r) / 2 + (r - l) / 2 * (cos( sw ));
		pntv.y = (t + b) / 2 - (b - t) / 2 * (sin( sw ));

		pDC->Arc( &rct, pntu, pntv );

		RESTOREPEN;
	}
}


void
mfcDraw::IPaintOval( int l, int t, int r, int b, int start, int sweep, rgb color )
{
	if ( ( l < r ) && ( t < b ) ) {
		SAVEBRUSH;
		SAVEPEN;

		RECT	rct = { l, t, r, b };
		POINT   pntu, pntv;

		double     st = 2 * pi * start / 360;
		double     sw = 2 * pi * (start + sweep) / 360;

		pntu.x = (l + r) / 2 + (r - l) / 2 * (cos( st ));
		pntu.y = (t + b) / 2 - (b - t) / 2 * (sin( st ));
		pntv.x = (l + r) / 2 + (r - l) / 2 * (cos( sw ));
		pntv.y = (t + b) / 2 - (b - t) / 2 * (sin( sw ));

		pDC->Pie( &rct, pntu, pntv );

		RESTOREBRUSH;
		RESTOREPEN;
	}
}


void
mfcDraw::IDrawPolygon( iPoint vertex[], int num_vertices, rgb color )
{
	if ( num_vertices > 0 ) {
		SAVEPEN;

		pDC->MoveTo( vertex[0].i, vertex[0].j );

		for ( int k = 1 ; k < num_vertices ; k++ )
			pDC->LineTo( vertex[k].i, vertex[k].j );

		pDC->LineTo( vertex[0].i, vertex[0].j );

		RESTOREPEN;
	}
}


void
mfcDraw::IPaintPolygon( iPoint vertex[], int num_vertices, rgb color )
{
	if ( num_vertices > 0 ) {
		POINT *pp=new POINT[num_vertices];
		SAVEBRUSH;

		for ( int k = 0 ; k < num_vertices ; k++ ){
			pp[k].x=vertex[k].i;
			pp[k].y=vertex[k].j;
		}

		pDC->BeginPath();
			pDC->Polygon(pp,num_vertices);
		pDC->EndPath();
		pDC->FillPath();
		pDC->AbortPath();

		delete[] pp;
		RESTOREBRUSH;
	}
}

void
mfcDraw::IDrawString( int i, int j, string str, int font_size, string font_name,
														int centering, rgb color )
{
	int			len;
	int         nOldMode=0;
	RECT		rect= {0, 0, 0, 0};

	CString		temp(str.begin());
	CFont		myFont;
	CFont		*pOldFont;

	len = font_name.length();

	pDC->SetBkMode( TRANSPARENT );				// ̔wi𓧖ɂ
	COLORREF oldcolor = pDC->SetTextColor(GetColor(color));	//̐F̑I
	myFont.CreatePointFont( 10*font_size, font_name.begin() );
	pOldFont = pDC->SelectObject(&myFont);

	pDC->DrawText( temp, &rect, DT_CALCRECT );	// eLXg̓lp`vZ

	rect.left = i - (rect.right) / 2;
	rect.top = j - font_size * 2 / 3;
	rect.right = i + (rect.right) ;				// ɗ]TĂ
	rect.bottom = j + font_size;				// ӂɗ]TĂ

	pDC->DrawText(temp, &rect, DT_LEFT);        // eLXg[ɑ

	pDC->SetBkMode(nOldMode);
	myFont.DeleteObject();
	pDC->SetTextColor(oldcolor);	//̐F̑I
	return;
}


COLORREF mfcDraw::GetColor(rgb &color)
{
	return RGB((BYTE)(color.red*255),(BYTE)(color.green*255),(BYTE)(color.blue*255));
}