(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     17731,        436]*)
(*NotebookOutlinePosition[     18374,        458]*)
(*  CellTagsIndexPosition[     18330,        454]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[BoxData[{
    \(initialmatching[mm_] := 
      Module[{n, e, edges, match = {}}, \n\t
        edges = Position[mm, _?\((Function[n, n != 0])\)]; \n\t
        e = First[edges]; \n\t
        While[First[e] < Last[e], \n\t\tIf[
            Not[MemberQ[Flatten[match], First[e]]]\  && \ 
              Not[MemberQ[Flatten[match], Last[e]]], 
            AppendTo[match, e]]; \n\t\tedges = Rest[edges]; 
          e = First[edges]\[IndentingNewLine]\ \ ]; \n\t
        Return[match]\n]\), "\n", 
    \(\(\(newmatching[match_, path_] := 
      Union[Complement[Union[match, path], 
          Intersection[match, path]]]\)\(\n\)
    \)\), "\n", 
    \(\(\(getmatchingedge[y_, mt_] := 
      First[mt[\([First[First[Position[Map[Last, mt], y]]]]\)]]\)\(\n\)
    \)\), "\n", 
    \(\(\(findneighbour[mm_, s_] := 
      Union[Flatten[
          Union[Map[Position[mm[\([#]\)], _?\((Function[n, n != 0])\)] &, 
              s]]]]\)\(\n\)
    \)\), "\[IndentingNewLine]", 
    \(\(\(searchparent[y_, s_, mm_] := \n\t
      Scan[\((If[mm[\([#, y]\)] == 1, Return[#]])\) &, 
        s]\)\(\[IndentingNewLine]\)
    \)\), "\n", 
    \(findaugmentingpath[tree_] := 
      Block[{start, tmp, topedge, endedge, path = {}}, \n\t
        If[Length[tree] == 1, Return[tree]]; \n\t
        start = First[First[tree]]; \n\t
        If[start == First[Last[tree]], Return[{Last[tree]}]]; \n\t
        topedge = \(endedge = First[Reverse[tree]]\); \n\t
        tmp = Rest[Reverse[tree]]; \n\t
        Scan[\((If[First[topedge] == Last[#], topedge = #; 
                AppendTo[path, #]; \n\t
                If[First[#] == start, path = Reverse[path]; \n\t
                  AppendTo[path, endedge]; Return[]]])\) &, tmp]; \n\t
        Return[Map[Sort, path]]\n]\), "\n", 
    \(hungarianmethod[m_] := 
      Module[\n (*\ 
          The\ complete\ bipartite\ graph, \ \(\(|\)\(x\)\(|\)\) = \(\(|\)\(Y\
\)\(|\)\)\ *) \n{i, n = Length[m], n1, xset, yset, ngl, s, t, lastx, path, 
          match}, \n\tn1 = n/2; \n\txset = Table[i, {i, n1}]; \n\t
        yset = Table[n1 + i, {i, n1}]; \n\tmatch = initialmatching[m]; \n\t
        While[Length[match] < n1, \n\t\ts = 
            Complement[xset, Map[First, match]]; \n\t\tlastx = First[s]; 
          path = {}; \n\t\ts = {lastx}; 
          t = {}; \n\t\tWhile[
            True, \n\t\t\tngl = findneighbour[m, s]; \n\t\t\tIf[
              Length[ngl] == Length[t], \n\t\t\t\tReturn[
                match]\n\t\t\t]; \[IndentingNewLine]\t\ty = 
              First[Complement[ngl, t]]; \[IndentingNewLine]\t\tIf[
              m[\([lastx, y]\)] == 1, \ \  (*\ 
                BugFix\ \(2002/10\)/10\ *) \n\t\t\t\tAppendTo[
                path, {lastx, y}], \n\t\t\t\tAppendTo[
                path, {searchparent[y, s, m], 
                  y}]\[IndentingNewLine]\t\ \ \ \ \ \ ]; \n\t\t\tIf[
              MemberQ[Map[Last, match], y], \n\t\t\t\tz = 
                getmatchingedge[y, match]; \n\t\t\t\tAppendTo[
                path, {y, z}]; \n\t\t\t\tlastx = z; \n\t\t\t\ts = 
                Append[s, z]; t = Append[t, y], 
              Break[]\n\t\t\t]\n\t\t]; \n\t\tpath = 
            findaugmentingpath[path];  (*\ 
            find\ a\ path\ in\ a\ tree\ *) \n\t\tmatch = 
            Union[Complement[Union[match, path], 
                Intersection[match, path]]];\n\t]; \n\t
        Return[match]\n]\)}], "Input"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"\:30b9\:30da\:30eb\
\:9593\:9055\:3044\:306e\:53ef\:80fd\:6027\:304c\:3042\:308a\:307e\:3059\:ff0e\
\:65b0\:898f\:30b7\:30f3\:30dc\:30eb\\\"\\!\\(yset\\)\\\"\:306f\:3059\:3067\
\:306b\:3042\:308b\:30b7\:30f3\:30dc\:30eb\\\"\\!\\(xset\\)\\\"\:306b\:4f3c\
\:3066\:3044\:307e\:3059\:ff0e \\!\\(\\*ButtonBox[\\\"\:8a73\:7d30\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message"]
}, Open  ]],

Cell[BoxData[{
    \(firstequalitygraph[m_, n_, n1_] := 
      Module[{i, j, lxy, egl}, \n\tegl = Table[0, {i, n}, {j, n}]; \n\t
        lxy = Map[Max, m]; \n\t
        For[i = 1, 
          i <= n1, \(i++\), \n\t\tlxy[\([i + n1]\)] = 0; \n\t\tFor[
            j = n1 + 1, 
            j <= n, \(j++\), \n\t\t\tIf[
              m[\([i, j]\)] == 
                lxy[\([i]\)], \n\t\t\t\tegl[\([i, j]\)] = \(egl[\([j, i]\)] = 
                  1\)]]]; \n\t{egl, lxy}\t\t\n]\), "\n", 
    \(\(\(getnewequalitygraph[m_, n1_, lxy_, s_, t_, ct_] := 
      Module[{i, j, k, al, b, si, lxy1, egl}, \n\tal = \[Infinity]; \n\t
        For[i = 1, 
          i <= Length[s], \(i++\), \n\t\tsi = s[\([i]\)]; \n\t\tFor[j = 1, 
            j <= Length[ct], \(j++\), \n\t\t\tb = 
              lxy[\([si]\)] + lxy[\([ct[\([j]\)]]\)] - 
                m[\([si, ct[\([j]\)]]\)\[IndentingNewLine]\ ]; \n\t\tIf[
              al > b, al = b]]\[IndentingNewLine]\ ]; \n\tlxy1 = lxy; \n\t
        For[i = 1, 
          i <= Length[s], \(i++\), \n\t\tlxy1[\([s[\([i]\)]]\)] = 
            lxy[\([s[\([i]\)]]\)] - al\[IndentingNewLine]\ ]; \n\t
        For[j = 1, 
          j <= Length[t], \(j++\), \n\t\tlxy1[\([t[\([j]\)]]\)] = 
            lxy[\([t[\([j]\)]]\)] + al\[IndentingNewLine]\ ]; \n\n\t
        egl = Table[0, {i, 2  n1}, {j, 2  n1}]; \n\t
        For[i = 1, 
          i <= n1, \(i++\), \n\t\tFor[k = 1 + n1, 
            k <= 2  n1, \(k++\), \n\t\t\tIf[
              m[\([i, k]\)] == 
                lxy1[\([i]\)] + 
                  lxy1[\([k]\)], \n\t\t\t\tegl[\([i, 
                    k]\)] = \(egl[\([k, i]\)] = 
                  1\)]\[IndentingNewLine]\t]\[IndentingNewLine]\ \ ]; \n\t\
{egl, lxy1}\t\t\t\t\n]\)\(\[IndentingNewLine]\)
    \)\), "\n", 
    \(\(\(kuhnmunkres[m_] := 
      Module[\n (*\ 
          The\ complete\ bipartite\ graph, \ \(\(|\)\(x\)\(|\)\) = \(\(|\)\(Y\
\)\(|\)\)\ *) \n{i, n = Length[m], n1, lxy, g, xset, yset, egl, \n\tngl, tmp, 
          s, t, path, match}, \n\tn1 = n/2; \n\txset = Table[i, {i, n1}]; \n\t
        yset = Table[n1 + i, {i, n1}]; \n\t
        tmp = firstequalitygraph[m, n, n1]; \n\tegl = tmp[\([1]\)]; \n\t
        lxy = tmp[\([2]\)]; \n\tmatch = hungarianmethod[egl]; \n\t
        While[Length[match] < n1, \n\t\ts = 
            Complement[xset, Map[First, match]]; \n\t\tlastx = First[s]; 
          path = {}; \n\t\ts = {lastx}; 
          t = {}; \n\t\tWhile[
            True, \n\t\t\tngl = findneighbour[egl, s]; \n\t\t\tIf[
              Length[ngl] == Length[t], \n\t\t\t\tct = 
                Complement[yset, t]; \n\t\t\t\ttmp = 
                getnewequalitygraph[m, n1, lxy, s, t, ct]; \n\t\t\t\tegl = 
                tmp[\([1]\)]; \n\t\t\t\tlxy = tmp[\([2]\)]; \n\t\t\t\tngl = 
                findneighbour[egl, s];\ \ \ \ \ \n\t\t\t]; \n\t\t\ty = 
              First[Complement[ngl, t]]; \n\t\t\tIf[
              egl[\([lastx, y]\)] == 1, \n\t\t\t\tAppendTo[
                path, {lastx, y}], \n\t\t\t\tAppendTo[
                path, {searchparent[y, s, egl], 
                  y}]\[IndentingNewLine]\t\ \ \ \ \ \ ]; \n\t\t\tIf[
              MemberQ[Map[Last, match], y], \n\t\t\t\tz = 
                getmatchingedge[y, match]; \n\t\t\t\tAppendTo[
                path, {y, z}]; \n\t\t\t\tlastx = z; \n\t\t\t\tAppendTo[s, z]; 
              AppendTo[t, y], Break[]\n\t\t\t]\n\t\t]; \n\t\tpath = 
            findaugmentingpath[
              path]; \[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \  (*\ 
            find\ a\ path\ in\ a\ tree, \ 
            in\ the\ case\ the\ last\ vertex\ is\ \(\(unsaturated\)\(.\)\)\ \
*) \n\t\tmatch = 
            Union[Complement[Union[match, path], 
                Intersection[match, path]]];\n\t]; \n\t
        Return[match]\n]\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(\(ConvertWMatrixToAMatrix[wm_List] := 
      Module[{i, ex, len, am = {}, zerolist, tr}, \[IndentingNewLine]len = 
          Length[wm]; \[IndentingNewLine]zerolist = 
          Table[0, {i, len}]; \[IndentingNewLine]For[i = 1, 
          i \[LessEqual] len, \(i++\), \[IndentingNewLine]ex = 
            Join[zerolist, wm[\([i]\)]]; \[IndentingNewLine]am = 
            Append[am, ex]]; \[IndentingNewLine]tr = 
          Transpose[wm]; \[IndentingNewLine]For[i = 1, 
          i \[LessEqual] len, \(i++\), \[IndentingNewLine]ex = 
            Join[tr[\([i]\)], zerolist]; \[IndentingNewLine]am = 
            Append[am, ex]]; \[IndentingNewLine]Return[
          am]]\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(KuhnMunkres[wm_List] := 
      Module[{mm, n1, match}, \[IndentingNewLine]n1 = 
          Length[wm]; \[IndentingNewLine]mm = 
          ConvertWMatrixToAMatrix[wm]; \[IndentingNewLine]match = 
          kuhnmunkres[
            mm]; \[IndentingNewLine]Print["\<The optimal cost = \>", \n\t\t\t\
Apply[Plus, 
            Map[mm[\([First[#], Last[#]]\)] &, 
              match]]]; \[IndentingNewLine]match = 
          Map[\(({First[#], Last[#] - n1})\) &, 
            match]; \[IndentingNewLine]Return[
          match]]\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(getmatching[lst_] := 
      Module[{i = 1}, \n\t\tMap[\(({\(i++\), #})\) &, 
          lst]\[IndentingNewLine]]\n\), "\[IndentingNewLine]", 
    \(OptimalMatching[wm_] := 
      Block[{mm, n1, lst, cost, maxcost, match, maxmatch, 
          matchlist}, \[IndentingNewLine]\ \ \ \ \ mm = 
          ConvertWMatrixToAMatrix[wm]; \n\tn1 = Length[mm]/2; \n\t
        lst = Table[n1 + i, {i, n1}]; \n\tmaxcost = 0; \n\t
        matchlist = Map[getmatching, Permutations[lst]]; \n\t
        While[matchlist != {}, \n\t\tmatch = First[matchlist]; \n\t\tcost = 
            Apply[Plus, Map[mm[\([First[#], Last[#]]\)] &, match]]; \n\t\tIf[
            cost > maxcost, maxcost = cost; 
            maxmatch = match]; \n\t\tmatchlist = 
            Rest[matchlist]\[IndentingNewLine]\ \ ]; \[IndentingNewLine]\ \ \ \
\ \ match = Map[\(({First[#], Last[#] - n1})\) &, maxmatch]; \n\t
        Return[{maxcost, match}]\n]\)}], "Input"],

Cell[BoxData[
    \(randombipartitematrix[n1_, p_, r_]\  := 
      Module[{i, n = 2  n1, mt}, \n\t
        mt\  = \ Table[
            If[i <= n1, 
              Join[Table[0, {n1}], 
                Table[\ \n\t\t\t\t\tIf[Random[Real] < p, 
                    Random[Integer, r], \ 0], \n\t\t\t\t\t{n1}]], 
              Join[Table[\ \n\t\t\t\t\tIf[Random[Real] < p, 
                    Random[Integer, r], \ 0], \n\t\t\t\t\t{n1}], 
                Table[0, {n1}]]], {i, n}\n\t\t]; \n\t\tmt + 
          Transpose[mt]\n\t]\)], "Input"],

Cell[BoxData[
    \(\(mm = {{0, \ 0, \ 0, \ 0, \ 0, \ 0, \ 1, \ 1, \ 1, \ 1}, \ \ \ {0, \ 
            0, \ 0, \ 0, \ 0, \ 0, \ 1, \ 0, \ 0, \ 0}, \ \ \n\ \ \ \ \ {0, \ 
            0, \ 0, \ 0, \ 0, \ 1, \ 0, \ 0, \ 0, \ 0}, \ \ \ {0, \ 0, \ 0, \ 
            0, \ 0, \ 0, \ 0, \ 1, \ 0, \ 0}, \ \ \n\ \ \ \ \ {0, \ 0, \ 0, \ 
            0, \ 0, \ 0, \ 0, \ 0, \ 0, \ 1}, \ \ {0, \ 0, \ 1, \ 0, \ 0, \ 
            0, \ 0, \ 0, \ 0, \ 0}, \ \ \n\ \ \ \ \ {1, \ 1, \ 0, \ 0, \ 0, \ 
            0, \ 0, \ 0, \ 0, \ 0}, \ \ {1, \ 0, \ 0, \ 1, \ 0, \ 0, \ 0, \ 
            0, \ 0, \ 0}, \ \n\ \ \ \ \ {1, \ 0, \ 0, \ 0, \ 0, \ 0, \ 0, \ 
            0, \ 0, \ 0}, \ {1, \ 0, \ 0, \ 0, \ 1, \ 0, \ 0, \ 0, \ 0, \ 
            0}};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(hungarianmethod[mm]\)], "Input"],

Cell[BoxData[
    \({{1, 9}, {2, 7}, {3, 6}, {4, 8}, {5, 10}}\)], "Output"]
}, Open  ]],

Cell[BoxData[{
    \(\(A = {{7, \ 14, \ 12, \ 8, \ 13}, \ \n\ \ \ \ \ \ \ \ \ {10, \ 17, \ 
            10, \ 8, \ 15}, \ \n\ \ \ \ \ \ \ \ \ {20, \ 11, \ 17, \ 3, \ 
            8}, \ \n\ \ \ \ \ \ \ \ \ {12, \ 13, \ 18, \ 13, \ 
            12}, \ \n\ \ \ \ \ \ \ \ \ {5, \ 12, \ 12, \ 11, \ 
            17}};\)\), "\[IndentingNewLine]", 
    \(\(mm = {{0, \ 0, \ 0, \ 0, \ 0, \ 7, \ 14, \ 12, \ 8, \ 
            13}, \ \n\ \ {0, \ 0, \ 0, \ 0, \ 0, \ 10, \ 17, \ 10, \ 8, \ 
            15}, \ \n\ \ {0, \ 0, \ 0, \ 0, \ 0, \ 20, \ 11, \ 17, \ 3, \ 
            8}, \ \n\ \ {0, \ 0, \ 0, \ 0, \ 0, \ 12, \ 13, \ 18, \ 13, \ 
            12}, \ \n\ \ {0, \ 0, \ 0, \ 0, \ 0, \ 5, \ 12, \ 12, \ 11, \ 
            17}, \ \n\ \ {7, \ 10, \ 20, \ 12, \ 5, \ 0, \ 0, \ 0, \ 0, \ 
            0}, \ \n\ \ {14, \ 17, \ 11, \ 13, \ 12, \ 0, \ 0, \ 0, \ 0, \ 
            0}, \ \n\ \ {12, \ 10, \ 17, \ 18, \ 12, \ 0, \ 0, \ 0, \ 0, \ 
            0}, \ \n\ \ {8, \ 8, \ 3, \ 13, \ 11, \ 0, \ 0, \ 0, \ 0, \ 
            0}, \ \n\ \ {13, \ 15, \ 8, \ 12, \ 17, \ 0, \ 0, \ 0, \ 0, \ 
            0}};\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kuhnmunkres[mm]\)], "Input"],

Cell[BoxData[
    \({{1, 9}, {2, 7}, {3, 6}, {4, 8}, {5, 10}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(OptimalMatching[A]\)], "Input"],

Cell[BoxData[
    \({80, {{1, 4}, {2, 2}, {3, 1}, {4, 3}, {5, 5}}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(mm = randombipartitematrix[20, 1, 100];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kuhnmunkres[mm]\)], "Input"],

Cell[BoxData[
    \({{1, 29}, {2, 40}, {3, 26}, {4, 28}, {5, 27}, {6, 30}, {7, 38}, {8, 
        35}, {9, 22}, {10, 36}, {11, 39}, {12, 33}, {13, 32}, {14, 21}, {15, 
        37}, {16, 31}, {17, 23}, {18, 24}, {19, 25}, {20, 34}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(mm = ConvertWMatrixToAMatrix[A];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(hungarianmethod[mm]\)], "Input"],

Cell[BoxData[
    \({{1, 6}, {2, 7}, {3, 8}, {4, 9}, {5, 10}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(OptimalMatching[A]\)], "Input"],

Cell[BoxData[
    \({80, {{1, 4}, {2, 2}, {3, 1}, {4, 3}, {5, 5}}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(A = {{0, 0, 2, 0, 2, 1}, \[IndentingNewLine]{4, 2, 1, 4, 2, 
            3}, \[IndentingNewLine]{3, 5, 3, 1, 5, 1}, \[IndentingNewLine]{0, 
            1, 1, 1, 0, 1}, \[IndentingNewLine]{6, 2, 4, 6, 6, 
            5}, \[IndentingNewLine]{1, 3, 3, 2, 3, 1}};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(KuhnMunkres[A]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("The optimal cost = "\[InvisibleSpace]21\),
      SequenceForm[ "The optimal cost = ", 21],
      Editable->False]], "Print"],

Cell[BoxData[
    \({{1, 5}, {2, 4}, {3, 2}, {4, 6}, {5, 1}, {6, 3}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(OptimalMatching[A]\)], "Input"],

Cell[BoxData[
    \({21, {{1, 3}, {2, 1}, {3, 2}, {4, 6}, {5, 4}, {6, 5}}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(A = {{3, 5, 5, 4, 1}, \[IndentingNewLine]{2, 2, 0, 2, 
            2}, \[IndentingNewLine]{2, 4, 4, 1, 0}, \[IndentingNewLine]{0, 1, 
            1, 0, 0}, \[IndentingNewLine]{1, 2, 1, 3, 3}};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(KuhnMunkres[A]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("The optimal cost = "\[InvisibleSpace]14\),
      SequenceForm[ "The optimal cost = ", 14],
      Editable->False]], "Print"],

Cell[BoxData[
    \({{1, 2}, {2, 4}, {3, 3}, {4, 1}, {5, 5}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(OptimalMatching[A]\)], "Input"],

Cell[BoxData[
    \({14, {{1, 2}, {2, 1}, {3, 3}, {4, 4}, {5, 5}}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(A = {{4, 5, 8, 10, 11}, \[IndentingNewLine]{7, 6, 5, 7, 
            4}, \[IndentingNewLine]{8, 5, 12, 9, 6}, \[IndentingNewLine]{6, 
            6, 13, 10, 7}, \[IndentingNewLine]{4, 5, 7, 9, 8}};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(KuhnMunkres[A]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("The optimal cost = "\[InvisibleSpace]47\),
      SequenceForm[ "The optimal cost = ", 47],
      Editable->False]], "Print"],

Cell[BoxData[
    \({{1, 5}, {2, 2}, {3, 1}, {4, 3}, {5, 4}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(OptimalMatching[A]\)], "Input"],

Cell[BoxData[
    \({47, {{1, 5}, {2, 2}, {3, 1}, {4, 3}, {5, 4}}}\)], "Output"]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 695}},
WindowSize->{496, 537},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 3365, 67, 1710, "Input"],
Cell[5144, 122, 518, 7, 52, "Message"]
}, Open  ]],
Cell[5677, 132, 6048, 112, 2730, "Input"],
Cell[11728, 246, 538, 11, 250, "Input"],
Cell[12269, 259, 744, 10, 210, "Input"],

Cell[CellGroupData[{
Cell[13038, 273, 52, 1, 30, "Input"],
Cell[13093, 276, 75, 1, 29, "Output"]
}, Open  ]],
Cell[13183, 280, 1112, 16, 310, "Input"],

Cell[CellGroupData[{
Cell[14320, 300, 48, 1, 30, "Input"],
Cell[14371, 303, 75, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14483, 309, 51, 1, 30, "Input"],
Cell[14537, 312, 80, 1, 29, "Output"]
}, Open  ]],
Cell[14632, 316, 76, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[14733, 321, 48, 1, 30, "Input"],
Cell[14784, 324, 243, 3, 86, "Output"]
}, Open  ]],
Cell[15042, 330, 69, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[15136, 335, 52, 1, 30, "Input"],
Cell[15191, 338, 75, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15303, 344, 51, 1, 30, "Input"],
Cell[15357, 347, 80, 1, 29, "Output"]
}, Open  ]],
Cell[15452, 351, 296, 4, 130, "Input"],

Cell[CellGroupData[{
Cell[15773, 359, 47, 1, 30, "Input"],
Cell[15823, 362, 163, 3, 25, "Print"],
Cell[15989, 367, 82, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16108, 373, 51, 1, 30, "Input"],
Cell[16162, 376, 88, 1, 29, "Output"]
}, Open  ]],
Cell[16265, 380, 229, 3, 110, "Input"],

Cell[CellGroupData[{
Cell[16519, 387, 47, 1, 30, "Input"],
Cell[16569, 390, 163, 3, 25, "Print"],
Cell[16735, 395, 74, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16846, 401, 51, 1, 30, "Input"],
Cell[16900, 404, 80, 1, 29, "Output"]
}, Open  ]],
Cell[16995, 408, 234, 3, 110, "Input"],

Cell[CellGroupData[{
Cell[17254, 415, 47, 1, 30, "Input"],
Cell[17304, 418, 163, 3, 25, "Print"],
Cell[17470, 423, 74, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17581, 429, 51, 1, 30, "Input"],
Cell[17635, 432, 80, 1, 29, "Output"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

